// Capacitance Meter
//
// CONNECT NEGATIVE (-) PIN OF CAP TO TEST PIN 0
// CONNECT POSITIVE (+) PIN OF CAP TO TEST PIN 1
/////////////////////////////////////////////////////

#include "ssd1306.h"

#define analogPin A1          
#define chargePin 6     // 470K to A1         
#define dischargePin 7  // 680R to A1       
#define resistorValue 470000.0F   
#define LeftButton 12

const PROGMEM uint8_t cap_bmp[] ={ 
  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,
  0x20, 0xFF, 0xFF, 0x00,  0x00, 0xFF, 0xFF, 0x20,
  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,

  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
  0x00, 0x07, 0x07, 0x00,  0x00, 0x07, 0x07, 0x00,
  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,

  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,

  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00
};

unsigned long startTime;
unsigned long elapsedTime;
float microFarads;                
float nanoFarads;
char strBuf[20];

void setup(){
  ssd1306_setFixedFont(ssd1306xled_font6x8);
  ssd1306_128x64_i2c_init();
  ssd1306_clearScreen();
    
  pinMode(A0, OUTPUT);    // Ground A0 Side of Capacitor
  digitalWrite(A0, LOW);
  
  pinMode(analogPin, INPUT);  // Measurement Side of Cap

  pinMode(LeftButton, INPUT_PULLUP);   
  pinMode(chargePin, OUTPUT);     
  digitalWrite(chargePin, LOW);  
  Serial.begin(9600);             
}

void loop(){
  ssd1306_clearScreen( );
  ssd1306_printFixed(0,  0, "Measuring...", STYLE_NORMAL); 
  
  digitalWrite(chargePin, HIGH);  
  startTime = millis();
  while(analogRead(analogPin) < 648){       
  }
  elapsedTime= millis() - startTime;
  microFarads = ((float)elapsedTime / resistorValue) * 1000;   
  Serial.print(elapsedTime);       
  Serial.print(" mS    ");         

  ssd1306_clearScreen( );
  ssd1306_printFixed(0,  0, "Capacitor", STYLE_NORMAL);
  ssd1306_drawBitmap(10, 10, 24, 32, cap_bmp);
  
  if (microFarads > 1){
    sprintf(strBuf, "%d microfarads", (long)microFarads);
    Serial.println(strBuf);   
    ssd1306_printFixed(0,  50, strBuf, STYLE_NORMAL);      
  }
  else{
    nanoFarads = microFarads * 1000.0;      
    sprintf(strBuf, "%d nanofarads", (long)nanoFarads);     
    Serial.println(strBuf);   
    ssd1306_printFixed(0,  50, strBuf, STYLE_NORMAL);      
  }

  digitalWrite(chargePin, LOW);            
  pinMode(dischargePin, OUTPUT);            
  digitalWrite(dischargePin, LOW);          
  while(analogRead(analogPin) > 0){         
  }

  pinMode(dischargePin, INPUT);     

  //Wait for Left Button to be pressed
  while(digitalRead(LeftButton) == HIGH)
    delay(50);   
} 
